module UU
  module OS
    module Server
      # Object representing server context.
      # @private
      class ServerContext # :nodoc:
        # Creates new instance of ServerContext.
        # @private
        def initialize(builder) # :nodoc:
          @builder = builder
          @@ctx = self
          @servlets = []
          @service_servlets = []
          @middlewares = []
        end

        # Returns true if server is running in development mode.
        # @return [TrueClass, FalseClass] True if running in development mode.
        def development_mode?
          @builder.development_mode?
        end

        def production_mode?
          @builder.production_mode?
        end

        def test_mode?
          @builder.test_mode?
        end

        def test_or_development_mode?
          @builder.test_or_development_mode?
        end

        # Returns true if server is running in development mode.
        # @return [TrueClass, FalseClass] True if running in development mode.
        def self.development_mode?
          @@ctx.development_mode?
        end

        def self.production_mode?
          @@ctx.production_mode?
        end

        def self.test_mode?
          @@ctx.test_mode?
        end

        def self.test_or_development_mode?
          @@ctx.test_or_development_mode?
        end

        # Registers servlet on given path.
        # @param [Class<UU::OS::Server::AbstractServlet>] servlet_class Servlet class.
        # @param [TrueClass, FalseClass] ignore_context Flag if server (development) context path should
        #    be ignored and given servlet should be registered into root context (defaults to nil/false).
        # @param [Array<String>] path Path where to register servlet (path elements are concatenated).
        # @return [void]
        def register_servlet(servlet_class, ignore_context = nil, *path)
          @builder.register_servlet(servlet_class, ignore_context, *path)
        end

        # @private
        def register_service_servlet(servlet_class, ignore_context = nil, *path) # :nodoc:
          @builder.register_service_servlet(servlet_class, ignore_context, *path)
        end

        # Returns server configuration.
        # @return [Hash<String, Object>] Server configuration.
        def configuration
          @builder.configuration
        end

        # Returns server configuration.
        # @return [Hash<String, Object>] Server configuration.
        def self.configuration
          @@ctx.configuration
        end

        # Returns time when server was started.
        # @return [Time] Time when server was started.
        def boot_time
          @builder.boot_time
        end

        # Returns list of registered servlets.
        # @return [Array<Hash>] List of servlet registrations (each
        #   registration has +:path+ and +:class+ attributes).
        def registered_servlets
          @builder.registered_servlets
        end

        # Returns absolute path to application's root directory
        #
        # @return [String]  Absolute path to application's root directory.
        def app_root_dir
          @builder.app_root_dir
        end

        # Registers servlet on given path.
        # @param [Class<UU::OS::Server::AbstractServlet>] servlet_class Servlet class.
        # @param [TrueClass, FalseClass] ignore_context Flag if server (development) context path should
        #    be ignored and given servlet should be registered into root context (defaults to nil/false).
        # @return [void]
        # @param [Array<String>] path Path where to register servlet (path elements are concatenated).
        def register_servlet(servlet_class, ignore_context = nil, *path)
          @servlets << [servlet_class, ignore_context, *path]
        end

        def register_service_servlet(servlet_class, ignore_context = nil, *path)
          @service_servlets << [servlet_class, ignore_context, *path]
        end

        def use_middleware(middleware, *args, &block)
          @middlewares << [middleware, *args, block]
        end

        def insert_middleware(position, middleware, *args, &block)
          @middlewares.insert(position, [middleware, *args, block])
        end

        # @private
        def middlewares
          @middlewares
        end

        # @private
        def servlets
          @servlets
        end

        # @private
        def service_servlets
          @service_servlets
        end

        # Override inspect to hide internal structures.
        # @private
        def inspect # :nodoc:
          "#<#{self.class.name}:0x#{(self.object_id << 1).to_s(16)}>"
        end

      end

    end
  end
end
